<?php
/**
 * Cette classe fournit une interface pour les notifications
 * @copyright   &copy; 2005-2019 PHPBoost
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL-3.0
 * @author      Kévin BRISSEZ <brissez.kevin@gmail.com>
 * @version     PHPBoost 5.1 - last update: 2019 11 01
 * @since       PHPBoost 5.0 - 2019 07 01
 */

class NotificationsArchivesController extends ModuleController
  {
    private $view;
    private $user;
    
    public function execute(HTTPRequestCustom $request)
      {
        
        $this->init();
        $this->build_view($request);
        
        $user = AppContext::get_current_user();
        
        if (!$user->check_level(User::MEMBER_LEVEL))
          {
            $error_controller = PHPBoostErrors::user_not_authorized();
            DispatchManager::redirect($error_controller);
          }
        
        // Si l'administrateur à activer la suppression automatique des notifications, alors il n'y a pas d'archive..
        if ($this->config->get_del_notif_after_view() == TRUE)
          {
            AppContext::get_response()->redirect(NotificationsUrlBuilder::home());
          }
        
        return $this->generate_response($request);
      }
    
    public function init()
      {
        $this->lang = LangLoader::get('common', 'notifications');
        $this->tpl  = new FileTemplate('notifications/archives.tpl');
        $this->tpl->add_lang($this->lang);
        $this->config = NotificationsConfig::load();
      }
    
    public function build_view(HTTPRequestCustom $request)
      {
        $user = AppContext::get_current_user();
        
        // On compte les notifications de l'utilisateur quand il est le destinataire 
        $nbr_archives = NotificationsService::count('WHERE recipient_id=:recip_id AND reading=1', array(
            'recip_id' => $user->get_id()
        ));
        
        // Variable pour la pagination qui fait appel à la fonction get_pagination, plus bas
        $page       = AppContext::get_request()->get_getint('page', 1);
        $pagination = $this->get_pagination($nbr_archives, $page);
        
        // S'il y a au moins une notification on en informe l'utilisateur.
        if ($nbr_archives > 0)
          {
            $list_archives = PersistenceContext::get_querier()->select('SELECT n.id AS id_notification, n.sender_id, n.recipient_id, n.content, n.date, n.reading, s.display_name AS username_sender, r.display_name AS username_recipient, a.user_avatar AS sender_avatar FROM ' . PREFIX . 'notifications n LEFT JOIN ' . PREFIX . 'member s ON s.user_id = n.sender_id LEFT JOIN ' . PREFIX . 'member r ON r.user_id = n.recipient_id LEFT JOIN ' . PREFIX . 'member_extended_fields a ON a.user_id = s.user_id WHERE n.recipient_id=:recip_id AND n.reading = 1 ORDER BY n.id DESC LIMIT :number_items_per_page OFFSET :display_from', array(
                'recip_id' => $user->get_id(),
                'number_items_per_page' => $pagination->get_number_items_per_page(),
                'display_from' => $pagination->get_display_from()
            ), SelectQueryResult::FETCH_ASSOC);
            
            
            // On fait une boucle qui liste toutes les archives de notifications de l'utilisateur quand Lu = 1
            while ($archives = $list_archives->fetch())
              {
                
                // Vérification de l'avatar, s'il est inexistant on le remplace par celui par défaut
                $user_accounts_config = UserAccountsConfig::load();
                if (empty($archives['sender_avatar']))
                  {
                    $avatar_sender = ($user_accounts_config->is_default_avatar_enabled() == '1') ? '<img src="' . PATH_TO_ROOT . '/templates/' . AppContext::get_current_user()->get_theme() . '/images/' . $user_accounts_config->get_default_avatar_name() . '" alt="' . LangLoader::get_message('avatar', 'user-common') . '" />' : '';
                  }
                else
                  {
                    $avatar_sender = '<img src="' . Url::to_rel($archives['sender_avatar']) . '" alt="' . LangLoader::get_message('avatar', 'user-common') . '"/>';
                  }
                
                $this->tpl->assign_block_vars('list_archives', array(
                    'ID' => $archives['id_notification'],
                    'CONTENT' => $archives['content'],
                    'SENDER_AVATAR' => $avatar_sender,
                    'DATE' => NotificationsFormatDate::getFormatDate('@' . $archives['date']),
                    'READ' => $archives['reading'] == 1 ? false : true,
                    'U_DELETE' => NotificationsUrlBuilder::delete($archives['id_notification'])->rel()
                ));
                
              }
            
          }
        
        $this->tpl->put_all(array(
            'C_PAGINATION' => $nbr_archives > NotificationsConfig::load()->get_items_per_page(),
            'PAGINATION' => $pagination->display(),
            'C_NO_ARCHIVES' => $nbr_archives == 0,
            'U_NOTIFICATION' => "/notifications",
            'U_DELETE_ALL' => "/notifications/archives/?del_all=1&token=" . AppContext::get_session()->get_token(),
            // DISPLAY_DEL_MARK affiche le menu de suppression et de "marquer comme vu" si l'admin a désactiver la suppression automatique
            'DISPLAY_DEL_MARK' => $this->config->get_del_notif_after_view() == true ? false : true,
            'L_MY_NOTIFICATION' => $this->lang['my.notifications'],
            'L_DELETE_ALL' => $this->lang['notifications.delete.all']
            
        ));
        
        // Suppression de toutes les archives
        $delete_all = $request->get_getint('del_all', 0);
        if (isset($delete_all) AND $delete_all == 1 AND $nbr_archives > 0 AND $this->config->get_del_notif_after_view() == FALSE)
          {
            // On génère un token pour éviter que quelqu'un nous envoi un lien qui efface nos archives
            AppContext::get_session()->csrf_get_protect();
            $recipient_id = $user->get_id();
            NotificationsService::delete('WHERE recipient_id=:id AND reading=1', array(
                'id' => $recipient_id
            ));
            AppContext::get_response()->redirect(($request->get_url_referrer() ? $request->get_url_referrer() : NotificationsUrlBuilder::archives()), LangLoader::get_message('notifications.delete.all.success', 'common', 'notifications'));
          }
        
        return $this->view;
        
      }
    
    private function get_pagination($nbr_archives, $page)
      {
        $pagination = new ModulePagination($page, $nbr_archives, (int) NotificationsConfig::load()->get_items_per_page());
        $pagination->set_url(NotificationsUrlBuilder::archives('%d'));
        
        if ($pagination->current_page_is_empty() && $page > 1)
          {
            // On préfère mettre cette fonction plutôt que de mettre une erreur 404, car en cas de suppression des notifications depuis la page 2 par exemple, ça nous aurais mis une erreur 404 en lien de retour...
            AppContext::get_response()->redirect(NotificationsUrlBuilder::archives());
          }
        
        return $pagination;
      }
    private function generate_response(HTTPRequestCustom $request)
      {
        $response = new SiteDisplayResponse($this->tpl);
        $response->get_graphical_environment()->set_page_title($this->lang['notifications.my.archives']);
        return $response;
      }
  }